@@ -1,14 +1,16 @@
 import logging
 import os
+import tempfile
 from contextlib import contextmanager
 
 from funcy import first
 from voluptuous import Invalid
 
 from dvc.cache.local import _log_exceptions
+from dvc.exceptions import DvcException
+from dvc.path_info import PathInfo
 from dvc.schema import COMPILED_LOCK_FILE_STAGE_SCHEMA
 from dvc.utils import dict_sha256, relpath
-from dvc.utils.fs import makedirs
 from dvc.utils.serialize import YAMLFileCorruptedError, dump_yaml, load_yaml
 
 from .loader import StageLoader
@@ -17,6 +19,11 @@ from .serialize import to_single_stage_lockfile
 logger = logging.getLogger(__name__)
 
 
+class RunCacheNotFoundError(DvcException):
+    def __init__(self, stage):
+        super().__init__(f"No run-cache for {stage.addressing}")
+
+
 def _get_cache_hash(cache, key=False):
     if key:
         cache["outs"] = [out["path"] for out in cache.get("outs", [])]
@@ -28,7 +35,7 @@ def _get_stage_hash(stage):
         return None
 
     for dep in stage.deps:
-        if not (dep.scheme == "local" and dep.def_path and dep.get_checksum()):
+        if not (dep.scheme == "local" and dep.def_path and dep.get_hash()):
             return None
 
     for out in stage.outs:
@@ -43,6 +50,10 @@ class StageCache:
         self.repo = repo
         self.cache_dir = os.path.join(repo.cache.local.cache_dir, "runs")
 
+    @property
+    def tree(self):
+        return self.repo.cache.local.tree
+
     def _get_cache_dir(self, key):
         return os.path.join(self.cache_dir, key[:2], key)
 
@@ -137,23 +148,42 @@ class StageCache:
         # sanity check
         COMPILED_LOCK_FILE_STAGE_SCHEMA(cache)
 
-        path = self._get_cache_path(cache_key, cache_value)
-        dpath = os.path.dirname(path)
-        makedirs(dpath, exist_ok=True)
-        dump_yaml(path, cache)
+        path = PathInfo(self._get_cache_path(cache_key, cache_value))
+        self.tree.makedirs(path.parent)
+        tmp = tempfile.NamedTemporaryFile(delete=False, dir=path.parent).name
+        assert os.path.exists(path.parent)
+        assert os.path.isdir(path.parent)
+        dump_yaml(tmp, cache)
+        self.tree.move(PathInfo(tmp), path)
 
-    def is_cached(self, stage):
-        return bool(self._load(stage))
-
-    def restore(self, stage):
+    def _restore(self, stage):
+        stage.save_deps()
         cache = self._load(stage)
         if not cache:
-            return
-        StageLoader.fill_from_lock(stage, cache)
+            raise RunCacheNotFoundError(stage)
 
+        StageLoader.fill_from_lock(stage, cache)
         for out in self._uncached_outs(stage, cache):
             out.checkout()
 
+        if not stage.outs_cached():
+            raise RunCacheNotFoundError(stage)
+
+    def restore(self, stage, run_cache=True):
+        if stage.is_callback or stage.always_changed:
+            raise RunCacheNotFoundError(stage)
+
+        if not stage.already_cached():
+            if not run_cache:  # backward compatibility
+                raise RunCacheNotFoundError(stage)
+            self._restore(stage)
+
+        logger.info(
+            "Stage '%s' is cached - skipping run, checking out outputs",
+            stage.addressing,
+        )
+        stage.checkout()
+
     @staticmethod
     def _transfer(func, from_remote, to_remote):
         ret = []
